package io.scanbot.sdk.reactnative.components.document_scanner_view

import android.Manifest
import android.app.Activity
import android.content.Context
import android.content.pm.PackageManager
import android.graphics.Color
import android.util.AttributeSet
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import androidx.core.app.ActivityCompat
import com.facebook.react.bridge.LifecycleEventListener
import com.facebook.react.bridge.ReactContext
import com.facebook.react.uimanager.UIManagerHelper
import io.scanbot.sdk.ScanbotSDK
import io.scanbot.sdk.camera.CaptureInfo
import io.scanbot.sdk.camera.FrameHandlerResult
import io.scanbot.sdk.core.contourdetector.DocumentDetectionStatus
import io.scanbot.sdk.docdetection.ui.DocumentScannerView
import io.scanbot.sdk.docdetection.ui.IDocumentFinderViewController
import io.scanbot.sdk.docdetection.ui.IDocumentScannerPolygonViewController
import io.scanbot.sdk.docdetection.ui.IDocumentScannerViewCallback
import io.scanbot.sdk.docdetection.ui.IDocumentScannerViewCameraConfiguration
import io.scanbot.sdk.docdetection.ui.IDocumentScannerViewController
import io.scanbot.sdk.process.ImageProcessor
import io.scanbot.sdk.reactnative.extensions.toPx
import io.scanbot.sdk.reactnative.models.BaseEvent
import io.scanbot.sdk.reactnative.models.ComponentEvent
import io.scanbot.sdk.reactnative.models.DocumentFinderViewDefaultConfiguration
import io.scanbot.sdk.reactnative.models.DocumentPolygonDefaultConfiguration
import io.scanbot.sdk.ui.camera.CameraUiSettings

class ScanbotDocumentScannerView(
    context: Context, attrs: AttributeSet?, defStyleAttr: Int, defStyleRes: Int
) : FrameLayout(context, attrs, defStyleAttr, defStyleRes) {

    constructor(context: Context) : this(context, null)
    constructor(context: Context, attrs: AttributeSet?) : this(context, attrs, 0)
    constructor(context: Context, attrs: AttributeSet?, defStyleAttr: Int) : this(
        context, attrs, defStyleAttr, 0
    )

    companion object {
        private const val CAMERA_PERMISSION_REQUEST_CODE = 3331
        private const val INITIAL_FLASH_STATE = false
    }

    private var contourDetector = ScanbotSDK(context).createContourDetector()
    lateinit var documentScanner: DocumentScannerView
        private set

    private var cameraPermissionRequested = false
    private var flashState: Boolean = INITIAL_FLASH_STATE

    private var lifecycleEventListener: LifecycleEventListener = object : LifecycleEventListener {
        override fun onHostResume() {
            requestCameraPermissionIfNotGranted()
            documentScanner.viewController.onResume()
        }

        override fun onHostPause() {
            documentScanner.viewController.onPause()
        }

        override fun onHostDestroy() {

        }
    }

    init {
        initDocumentScannerView()

        layoutParams = ViewGroup.LayoutParams(
            ViewGroup.LayoutParams.MATCH_PARENT,
            ViewGroup.LayoutParams.MATCH_PARENT,
        )
        setBackgroundColor(Color.BLACK)
        addView(documentScanner)
    }

    private fun initDocumentScannerView() {

        documentScanner = DocumentScannerView(context).apply {
            initCamera(CameraUiSettings(true))
            initDetectionBehavior(contourDetector, {
                (it as? FrameHandlerResult.Success)?.value?.let { result ->
                    sendEvent(DocumentScannerDetectionResultEvent(result))
                }

                false
            }, object : IDocumentScannerViewCallback {
                override fun onCameraOpen() {
                    viewController.useFlash(flashState)
                }

                override fun onPictureTaken(image: ByteArray, captureInfo: CaptureInfo) {
                    processTakenPicture(image, captureInfo.imageOrientation)
                    post {
                        viewController.startPreview()
                    }
                }
            })

            (context as ReactContext).addLifecycleEventListener(lifecycleEventListener)
            configureFinderViewControllerWithDefaults(finderViewController)
            configurePolygonConfigurationWithDefaults(polygonConfiguration)
            configureViewControllerDefaults(viewController)
            configureCameraConfigurationDefaults(cameraConfiguration)
        }
    }


    private fun configureFinderViewControllerWithDefaults(finderViewController: IDocumentFinderViewController) {
        finderViewController.setFinderEnabled(false)
        finderViewController.setStrokeWidth(2.0.toPx(this.resources))
        finderViewController.setFinderMinPadding(20.toPx(this.resources))
        finderViewController.setStrokeColor(DocumentFinderViewDefaultConfiguration.DEFAULT_FINDER_LINE_COLOR)
        finderViewController.setOverlayColor(DocumentFinderViewDefaultConfiguration.DEFAULT_OVERLAY_COLOR)
        finderViewController.setRequiredAspectRatios(DocumentFinderViewDefaultConfiguration.DEFAULT_ASPECT_RATIO)
    }

    private fun configurePolygonConfigurationWithDefaults(polygonConfiguration: IDocumentScannerPolygonViewController) {
        polygonConfiguration.setPolygonViewVisible(true)
        polygonConfiguration.setPolygonStrokeWidth(3.toPx(this.resources))
        polygonConfiguration.setPolygonCornerRadius(8.toPx(this.resources))
        polygonConfiguration.setPolygonAutoSnappingProgressStrokeWidth(5.toPx(this.resources))
        polygonConfiguration.setPolygonAutoSnapProgressEnabled(true)
        polygonConfiguration.setPolygonFillColor(DocumentPolygonDefaultConfiguration.DEFAULT_POLYGON_BACKGROUND_COLOR)
        polygonConfiguration.setPolygonFillColorOK(DocumentPolygonDefaultConfiguration.DEFAULT_POLYGON_BACKGROUND_COLOR_OK)
        polygonConfiguration.setPolygonStrokeColor(DocumentPolygonDefaultConfiguration.DEFAULT_POLYGON_COLOR)
        polygonConfiguration.setPolygonStrokeColorOK(DocumentPolygonDefaultConfiguration.DEFAULT_POLYGON_COLOR)
        polygonConfiguration.setPolygonAutoSnappingProgressStrokeColor(
            DocumentPolygonDefaultConfiguration.DEFAULT_POLYGON_AUTO_SNAP_COLOR
        )
    }

    private fun configureViewControllerDefaults(viewController: IDocumentScannerViewController) {
        viewController.isFrameProcessingEnabled = true
        viewController.setAcceptedAngleScore(75.0)
        viewController.setAcceptedBrightnessThreshold(50.0)
        viewController.setAcceptedSizeScore(80.0)
        viewController.autoSnappingEnabled = true
        viewController.setAutoSnappingSensitivity(0.66f)
        viewController.setIgnoreBadAspectRatio(false)
        viewController.setRequiredAspectRatios(emptyList())
    }

    private fun configureCameraConfigurationDefaults(cameraConfiguration: IDocumentScannerViewCameraConfiguration) {
        cameraConfiguration.setAutoFocusOnTouch(true)
    }

    private fun processTakenPicture(image: ByteArray, imageOrientation: Int) {
        val detectionResult = contourDetector.detect(image)
        if (detectionResult != null && detectionResult.status != DocumentDetectionStatus.ERROR_NOTHING_DETECTED) {
            val detectedPolygon = detectionResult.polygonF

            ImageProcessor(image).crop(detectedPolygon).let { processor ->
                if (imageOrientation > 0) {
                    processor.rotate(imageOrientation)
                }

                processor.encodedProcessedImage(".jpeg")?.let {
                    sendEvent(DocumentScannerResultEvent(it))
                }
            }
        } else {
            sendEvent(DocumentScannerResultEvent(image))
        }
    }

    private fun sendEvent(event: BaseEvent) {
        val surfaceId = UIManagerHelper.getSurfaceId(this)

        UIManagerHelper.getEventDispatcherForReactTag((context as ReactContext), id)
            ?.dispatchEvent(ComponentEvent(event, surfaceId, id))
    }

    override fun requestLayout() {
        super.requestLayout()
        post {
            layoutView()
        }
    }

    private fun View.layoutView() {
        measure(
            MeasureSpec.makeMeasureSpec(measuredWidth, MeasureSpec.EXACTLY),
            MeasureSpec.makeMeasureSpec(measuredHeight, MeasureSpec.EXACTLY)
        )
        layout(left, top, right, bottom)
    }

    override fun onAttachedToWindow() {
        super.onAttachedToWindow()
        this.documentScanner.viewController.onResume()
    }

    override fun onDetachedFromWindow() {
        this.documentScanner.viewController.onPause()
        super.onDetachedFromWindow()
    }

    fun removeAttachedListeners() {
        (context as ReactContext).removeLifecycleEventListener(lifecycleEventListener)
    }

    private fun requestCameraPermissionIfNotGranted() {
        val activity: Activity = (context as ReactContext).currentActivity ?: return

        if (!cameraPermissionRequested &&
            ActivityCompat.checkSelfPermission(
                activity,
                Manifest.permission.CAMERA
            ) != PackageManager.PERMISSION_GRANTED
        ) {
            // shouldShowRequestPermissionRationale can be changed from another place in the app,
            // that's why we are using custom flag
            cameraPermissionRequested = true

            ActivityCompat.requestPermissions(
                activity,
                arrayOf(Manifest.permission.CAMERA),
                CAMERA_PERMISSION_REQUEST_CODE
            )
        }
    }

}
